@@ -22,7 +22,7 @@
 *Description*
 
 This strategy migrates a VM based on the VM workload of the hosts.
-It makes decision to migrate a workload whenever a host's CPU
+It makes decision to migrate a workload whenever a host's CPU or RAM
 utilization % is higher than the specified threshold. The VM to
 be moved should make the host close to average workload of all
 hosts nodes.
@@ -32,7 +32,7 @@ hosts nodes.
 * Hardware: compute node should use the same physical CPUs
 * Software: Ceilometer component ceilometer-agent-compute
   running in each compute node, and Ceilometer API can
-  report such telemetry "cpu_util" successfully.
+  report such telemetry "cpu_util" and "memory.resident" successfully.
 * You must have at least 2 physical compute nodes to run
   this strategy.
 
@@ -46,15 +46,13 @@ hosts nodes.
   algorithm with `CONTINUOUS` audits.
 """
 
-import datetime
+from __future__ import division
 
 from oslo_config import cfg
 from oslo_log import log
 
 from watcher._i18n import _
 from watcher.common import exception as wexc
-from watcher.datasource import ceilometer as ceil
-from watcher.datasource import gnocchi as gnoc
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -68,16 +66,16 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
 
         It is a migration strategy based on the VM workload of physical
         servers. It generates solutions to move a workload whenever a server's
-        CPU utilization % is higher than the specified threshold.
+        CPU or RAM utilization % is higher than the specified threshold.
         The VM to be moved should make the host close to average workload
         of all compute nodes.
 
     *Requirements*
 
-        * Hardware: compute node should use the same physical CPUs
+        * Hardware: compute node should use the same physical CPUs/RAMs
         * Software: Ceilometer component ceilometer-agent-compute running
           in each compute node, and Ceilometer API can report such telemetry
-          "cpu_util" successfully.
+          "cpu_util" and "memory.resident" successfully.
         * You must have at least 2 physical compute nodes to run this strategy
 
     *Limitations*
@@ -90,8 +88,14 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
     """
 
     # The meter to report CPU utilization % of VM in ceilometer
-    METER_NAME = "cpu_util"
     # Unit: %, value range is [0 , 100]
+    CPU_METER_NAME = "cpu_util"
+
+    # The meter to report memory resident of VM in ceilometer
+    # Unit: MB
+    MEM_METER_NAME = "memory.resident"
+
+    DATASOURCE_METRICS = ['instance_cpu_usage', 'instance_ram_usage']
 
     MIGRATION = "migrate"
 
@@ -103,31 +107,9 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         :param osc: :py:class:`~.OpenStackClients` instance
         """
         super(WorkloadBalance, self).__init__(config, osc)
-        # the migration plan will be triggered when the CPU utlization %
-        # reaches threshold
-        self._meter = self.METER_NAME
-        self._ceilometer = None
-        self._gnocchi = None
-
-    @property
-    def ceilometer(self):
-        if self._ceilometer is None:
-            self._ceilometer = ceil.CeilometerHelper(osc=self.osc)
-        return self._ceilometer
-
-    @ceilometer.setter
-    def ceilometer(self, c):
-        self._ceilometer = c
-
-    @property
-    def gnocchi(self):
-        if self._gnocchi is None:
-            self._gnocchi = gnoc.GnocchiHelper(osc=self.osc)
-        return self._gnocchi
-
-    @gnocchi.setter
-    def gnocchi(self, gnocchi):
-        self._gnocchi = gnocchi
+        # the migration plan will be triggered when the CPU or RAM
+        # utilization % reaches threshold
+        self._meter = None
 
     @classmethod
     def get_name(cls):
@@ -150,6 +132,13 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         # Mandatory default setting for each element
         return {
             "properties": {
+                "metrics": {
+                    "description": "Workload balance based on metrics: "
+                                   "cpu or ram utilization",
+                    "type": "string",
+                    "choice": ["cpu_util", "memory.resident"],
+                    "default": "cpu_util"
+                },
                 "threshold": {
                     "description": "workload threshold for migration",
                     "type": "number",
@@ -172,13 +161,23 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
     @classmethod
     def get_config_opts(cls):
         return [
-            cfg.StrOpt(
-                "datasource",
-                help="Data source to use in order to query the needed metrics",
-                default="ceilometer",
-                choices=["ceilometer", "gnocchi"])
+            cfg.ListOpt(
+                "datasources",
+                help="Datasources to use in order to query the needed metrics."
+                     " If one of strategy metric isn't available in the first"
+                     " datasource, the next datasource will be chosen.",
+                item_type=cfg.types.String(choices=['gnocchi', 'ceilometer',
+                                                    'monasca']),
+                default=['gnocchi', 'ceilometer', 'monasca'])
         ]
 
+    def get_available_compute_nodes(self):
+        default_node_scope = [element.ServiceState.ENABLED.value]
+        return {uuid: cn for uuid, cn in
+                self.compute_model.get_all_compute_nodes().items()
+                if cn.state == element.ServiceState.ONLINE.value and
+                cn.status in default_node_scope}
+
     def calculate_used_resource(self, node):
         """Calculate the used vcpus, memory and disk based on VM flavors"""
         instances = self.compute_model.get_node_instances(node)
@@ -250,18 +249,21 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
             cores_available = host.vcpus - cores_used
             disk_available = host.disk - disk_used
             mem_available = host.memory - mem_used
-            if (
-                    cores_available >= required_cores and
-                    disk_available >= required_disk and
+            if (cores_available >= required_cores and
                     mem_available >= required_mem and
+                    disk_available >= required_disk):
+                if (self._meter == self.CPU_METER_NAME and
                     ((src_instance_workload + workload) <
-                     self.threshold / 100 * host.vcpus)
-            ):
-                destination_hosts.append(instance_data)
+                     self.threshold / 100 * host.vcpus)):
+                    destination_hosts.append(instance_data)
+                if (self._meter == self.MEM_METER_NAME and
+                    ((src_instance_workload + workload) <
+                     self.threshold / 100 * host.memory)):
+                    destination_hosts.append(instance_data)
 
         return destination_hosts
 
-    def group_hosts_by_cpu_util(self):
+    def group_hosts_by_cpu_or_ram_util(self):
         """Calculate the workloads of each node
 
         try to find out the nodes which have reached threshold
@@ -270,7 +272,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         and also generate the instance workload map.
         """
 
-        nodes = self.compute_model.get_all_compute_nodes()
+        nodes = self.get_available_compute_nodes()
         cluster_size = len(nodes)
         if not nodes:
             raise wexc.ClusterEmpty()
@@ -285,45 +287,39 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
             instances = self.compute_model.get_node_instances(node)
             node_workload = 0.0
             for instance in instances:
-                cpu_util = None
+                util = None
                 try:
-                    if self.config.datasource == "ceilometer":
-                        cpu_util = self.ceilometer.statistic_aggregation(
-                            resource_id=instance.uuid,
-                            meter_name=self._meter,
-                            period=self._period,
-                            aggregate='avg')
-                    elif self.config.datasource == "gnocchi":
-                        stop_time = datetime.datetime.utcnow()
-                        start_time = stop_time - datetime.timedelta(
-                            seconds=int(self._period))
-                        cpu_util = self.gnocchi.statistic_aggregation(
-                            resource_id=instance.uuid,
-                            metric=self._meter,
-                            granularity=self.granularity,
-                            start_time=start_time,
-                            stop_time=stop_time,
-                            aggregation='mean'
-                        )
+                    util = self.datasource_backend.statistic_aggregation(
+                        instance.uuid, self._meter, self._period, 'mean',
+                        granularity=self.granularity)
                 except Exception as exc:
                     LOG.exception(exc)
-                    LOG.error("Can not get cpu_util from %s",
+                    LOG.error("Can not get %s from %s", self._meter,
                               self.config.datasource)
                     continue
-                if cpu_util is None:
-                    LOG.debug("Instance (%s): cpu_util is None", instance.uuid)
+                if util is None:
+                    LOG.debug("Instance (%s): %s is None",
+                              instance.uuid, self._meter)
                     continue
-                workload_cache[instance.uuid] = cpu_util * instance.vcpus / 100
+                if self._meter == self.CPU_METER_NAME:
+                    workload_cache[instance.uuid] = (util *
+                                                     instance.vcpus / 100)
+                else:
+                    workload_cache[instance.uuid] = util
                 node_workload += workload_cache[instance.uuid]
-                LOG.debug("VM (%s): cpu_util %f", instance.uuid, cpu_util)
-            node_cpu_util = node_workload / node.vcpus * 100
+                LOG.debug("VM (%s): %s %f", instance.uuid, self._meter,
+                          util)
 
             cluster_workload += node_workload
+            if self._meter == self.CPU_METER_NAME:
+                node_util = node_workload / node.vcpus * 100
+            else:
+                node_util = node_workload / node.memory * 100
 
             instance_data = {
-                'node': node, "cpu_util": node_cpu_util,
+                'node': node, self._meter: node_util,
                 'workload': node_workload}
-            if node_cpu_util >= self.threshold:
+            if node_util >= self.threshold:
                 # mark the node to release resources
                 overload_hosts.append(instance_data)
             else:
@@ -355,8 +351,9 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         """
         self.threshold = self.input_parameters.threshold
         self._period = self.input_parameters.period
+        self._meter = self.input_parameters.metrics
         source_nodes, target_nodes, avg_workload, workload_cache = (
-            self.group_hosts_by_cpu_util())
+            self.group_hosts_by_cpu_or_ram_util())
 
         if not source_nodes:
             LOG.debug("No hosts require optimization")
@@ -372,7 +369,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
         # choose the server with largest cpu_util
         source_nodes = sorted(source_nodes,
                               reverse=True,
-                              key=lambda x: (x[self.METER_NAME]))
+                              key=lambda x: (x[self._meter]))
 
         instance_to_migrate = self.choose_instance_to_migrate(
             source_nodes, avg_workload, workload_cache)
@@ -390,7 +387,7 @@ class WorkloadBalance(base.WorkloadStabilizationBaseStrategy):
                         "be because of there's no enough CPU/Memory/DISK")
             return self.solution
         destination_hosts = sorted(destination_hosts,
-                                   key=lambda x: (x["cpu_util"]))
+                                   key=lambda x: (x[self._meter]))
         # always use the host with lowerest CPU utilization
         mig_destination_node = destination_hosts[0]['node']
         # generate solution to migrate the instance to the dest server,
