@@ -28,7 +28,6 @@ It assumes that live migrations are possible in your cluster.
 """
 
 import copy
-import datetime
 import itertools
 import math
 import random
@@ -41,8 +40,6 @@ import oslo_utils
 
 from watcher._i18n import _
 from watcher.common import exception
-from watcher.datasource import ceilometer as ceil
-from watcher.datasource import gnocchi as gnoc
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -65,6 +62,9 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
     MIGRATION = "migrate"
     MEMOIZE = _set_memoize(CONF)
 
+    DATASOURCE_METRICS = ['host_cpu_usage', 'instance_cpu_usage',
+                          'instance_ram_usage', 'host_memory_usage']
+
     def __init__(self, config, osc=None):
         """Workload Stabilization control using live migration
 
@@ -73,9 +73,6 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         :param osc: :py:class:`~.OpenStackClients` instance
         """
         super(WorkloadStabilization, self).__init__(config, osc)
-        self._ceilometer = None
-        self._gnocchi = None
-        self._nova = None
         self.weights = None
         self.metrics = None
         self.thresholds = None
@@ -169,43 +166,16 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
     @classmethod
     def get_config_opts(cls):
         return [
-            cfg.StrOpt(
-                "datasource",
-                help="Data source to use in order to query the needed metrics",
-                default="ceilometer",
-                choices=["ceilometer", "gnocchi"])
+            cfg.ListOpt(
+                "datasources",
+                help="Datasources to use in order to query the needed metrics."
+                     " If one of strategy metric isn't available in the first"
+                     " datasource, the next datasource will be chosen.",
+                item_type=cfg.types.String(choices=['gnocchi', 'ceilometer',
+                                                    'monasca']),
+                default=['gnocchi', 'ceilometer', 'monasca'])
         ]
 
-    @property
-    def ceilometer(self):
-        if self._ceilometer is None:
-            self.ceilometer = ceil.CeilometerHelper(osc=self.osc)
-        return self._ceilometer
-
-    @property
-    def nova(self):
-        if self._nova is None:
-            self.nova = self.osc.nova()
-        return self._nova
-
-    @nova.setter
-    def nova(self, n):
-        self._nova = n
-
-    @ceilometer.setter
-    def ceilometer(self, c):
-        self._ceilometer = c
-
-    @property
-    def gnocchi(self):
-        if self._gnocchi is None:
-            self.gnocchi = gnoc.GnocchiHelper(osc=self.osc)
-        return self._gnocchi
-
-    @gnocchi.setter
-    def gnocchi(self, gnocchi):
-        self._gnocchi = gnocchi
-
     def transform_instance_cpu(self, instance_load, host_vcpus):
         """Transform instance cpu utilization to overall host cpu utilization.
 
@@ -227,32 +197,15 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         LOG.debug('get_instance_load started')
         instance_load = {'uuid': instance.uuid, 'vcpus': instance.vcpus}
         for meter in self.metrics:
-            avg_meter = None
-            if self.config.datasource == "ceilometer":
-                avg_meter = self.ceilometer.statistic_aggregation(
-                    resource_id=instance.uuid,
-                    meter_name=meter,
-                    period=self.periods['instance'],
-                    aggregate='min'
-                )
-            elif self.config.datasource == "gnocchi":
-                stop_time = datetime.datetime.utcnow()
-                start_time = stop_time - datetime.timedelta(
-                    seconds=int(self.periods['instance']))
-                avg_meter = self.gnocchi.statistic_aggregation(
-                    resource_id=instance.uuid,
-                    metric=meter,
-                    granularity=self.granularity,
-                    start_time=start_time,
-                    stop_time=stop_time,
-                    aggregation='mean'
-                )
+            avg_meter = self.datasource_backend.statistic_aggregation(
+                instance.uuid, meter, self.periods['instance'], 'mean',
+                granularity=self.granularity)
             if avg_meter is None:
                 LOG.warning(
                     "No values returned by %(resource_id)s "
                     "for %(metric_name)s" % dict(
                         resource_id=instance.uuid, metric_name=meter))
-                avg_meter = 0
+                return
             if meter == 'cpu_util':
                 avg_meter /= float(100)
             instance_load[meter] = avg_meter
@@ -287,33 +240,15 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                     resource_id = "%s_%s" % (node.uuid, node.hostname)
                 else:
                     resource_id = node_id
-                if self.config.datasource == "ceilometer":
-                    avg_meter = self.ceilometer.statistic_aggregation(
-                        resource_id=resource_id,
-                        meter_name=self.instance_metrics[metric],
-                        period=self.periods['node'],
-                        aggregate='avg'
-                    )
-                elif self.config.datasource == "gnocchi":
-                    stop_time = datetime.datetime.utcnow()
-                    start_time = stop_time - datetime.timedelta(
-                        seconds=int(self.periods['node']))
-                    avg_meter = self.gnocchi.statistic_aggregation(
-                        resource_id=resource_id,
-                        metric=self.instance_metrics[metric],
-                        granularity=self.granularity,
-                        start_time=start_time,
-                        stop_time=stop_time,
-                        aggregation='mean'
-                    )
+                avg_meter = self.datasource_backend.statistic_aggregation(
+                    resource_id, self.instance_metrics[metric],
+                    self.periods['node'], 'mean', granularity=self.granularity)
 
                 if avg_meter is None:
-                    if meter_name == 'hardware.memory.used':
-                        avg_meter = node.memory
-                    if meter_name == 'compute.node.cpu.percent':
-                        avg_meter = 1
                     LOG.warning('No values returned by node %s for %s',
                                 node_id, meter_name)
+                    del hosts_load[node_id]
+                    break
                 else:
                     if meter_name == 'hardware.memory.used':
                         avg_meter /= oslo_utils.units.Ki
@@ -362,6 +297,8 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         migration_case = []
         new_hosts = copy.deepcopy(hosts)
         instance_load = self.get_instance_load(instance)
+        if not instance_load:
+            return
         s_host_vcpus = new_hosts[src_node.uuid]['vcpus']
         d_host_vcpus = new_hosts[dst_node.uuid]['vcpus']
         for metric in self.metrics:
@@ -379,6 +316,16 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         migration_case.append(new_hosts)
         return migration_case
 
+    def get_current_weighted_sd(self, hosts_load):
+        """Calculate current weighted sd"""
+        current_sd = []
+        normalized_load = self.normalize_hosts_load(hosts_load)
+        for metric in self.metrics:
+            metric_sd = self.get_sd(normalized_load, metric)
+            current_sd.append(metric_sd)
+        current_sd.append(hosts_load)
+        return self.calculate_weighted_sd(current_sd[:-1])
+
     def simulate_migrations(self, hosts):
         """Make sorted list of pairs instance:dst_host"""
         def yield_nodes(nodes):
@@ -393,14 +340,15 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                     yield nodes
 
         instance_host_map = []
-        nodes = list(self.get_available_nodes())
+        nodes = sorted(list(self.get_available_nodes()))
+        current_weighted_sd = self.get_current_weighted_sd(hosts)
         for src_host in nodes:
             src_node = self.compute_model.get_node_by_uuid(src_host)
             c_nodes = copy.copy(nodes)
             c_nodes.remove(src_host)
             node_list = yield_nodes(c_nodes)
             for instance in self.compute_model.get_node_instances(src_node):
-                min_sd_case = {'value': len(self.metrics)}
+                min_sd_case = {'value': current_weighted_sd}
                 if instance.state not in [element.InstanceState.ACTIVE.value,
                                           element.InstanceState.PAUSED.value]:
                     continue
@@ -408,6 +356,8 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                     dst_node = self.compute_model.get_node_by_uuid(dst_host)
                     sd_case = self.calculate_migration_case(
                         hosts, instance, src_node, dst_node)
+                    if sd_case is None:
+                        break
 
                     weighted_sd = self.calculate_weighted_sd(sd_case[:-1])
 
@@ -416,6 +366,8 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                             'host': dst_node.uuid, 'value': weighted_sd,
                             's_host': src_node.uuid, 'instance': instance.uuid}
                         instance_host_map.append(min_sd_case)
+                if sd_case is None:
+                    continue
         return sorted(instance_host_map, key=lambda x: x['value'])
 
     def check_threshold(self):
@@ -424,7 +376,12 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         normalized_load = self.normalize_hosts_load(hosts_load)
         for metric in self.metrics:
             metric_sd = self.get_sd(normalized_load, metric)
+            LOG.info("Standard deviation for %s is %s."
+                     % (metric, metric_sd))
             if metric_sd > float(self.thresholds[metric]):
+                LOG.info("Standard deviation of %s exceeds"
+                         " appropriate threshold %s."
+                         % (metric, metric_sd))
                 return self.simulate_migrations(hosts_load)
 
     def add_migration(self,
