@@ -27,7 +27,7 @@ from watcher.decision_engine.model import model_root as modelroot
 
 class FakerModelCollector(base.BaseClusterDataModelCollector):
 
-    def __init__(self, config=None, osc=None):
+    def __init__(self, config=None, osc=None, audit_scope=None):
         if config is None:
             config = mock.Mock()
         super(FakerModelCollector, self).__init__(config)
@@ -36,6 +36,9 @@ class FakerModelCollector(base.BaseClusterDataModelCollector):
     def notification_endpoints(self):
         return []
 
+    def get_audit_scope_handler(self, audit_scope):
+        return None
+
     def execute(self):
         return self.generate_scenario_1()
 
